@@ -1,6 +1,6 @@
 (* List CPU stats for a domain.
  * Usage: get_cpu_stats domain
- * http://libvirt.org/
+ * https://libvirt.org/
  *)
 
 open Printf
@@ -17,11 +17,13 @@ let () =
     );
     let domname = Sys.argv.(1) in
 
-    let conn = C.connect_readonly () in
+    let conn = C.connect_auth_readonly (C.get_auth_default ()) in
 
+    (*
     let nr_pcpus =
       let info = C.get_node_info conn in
       C.maxcpus_of_node_info info in
+     *)
 
     let stats =
       let dom = D.lookup_by_name conn domname in
