@@ -1,7 +1,7 @@
 (* Simple demo program showing node info.
    Usage: node_info [URI]
    (C) Copyright 2007 Richard W.M. Jones, Red Hat Inc.
-   http://libvirt.org/
+   https://libvirt.org/
  *)
 
 open Printf
@@ -15,7 +15,7 @@ let () =
 	Some (Sys.argv.(1))
       else
 	None in
-    let conn = C.connect_readonly ?name () in
+    let conn = C.connect_auth_readonly ?name (C.get_auth_default ()) in
 
     (* Get node_info, hostname, etc. *)
     let node_info = C.get_node_info conn in
